import React, { useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { formatDate } from '../../../../utils/dateUtils';
import { deleteKnowledgeBaseDocument } from '../../../../store/knowledgeBase/knowledgeBase.thunks';
import DocumentPreviewModal from './DocumentPreviewModal';

/**
 * 知识库文档列表组件
 */
const DocumentList = ({ knowledgeBaseId }) => {
    const dispatch = useDispatch();
    const { items, loading, pagination } = useSelector((state) => state.knowledgeBase.documents);
    const [previewModalVisible, setPreviewModalVisible] = useState(false);
    const [selectedDocumentId, setSelectedDocumentId] = useState(null);

    const handleDeleteDocument = (documentId) => {
        if (window.confirm('确定要删除此文档吗？')) {
            dispatch(
                deleteKnowledgeBaseDocument({
                    knowledge_base_id: knowledgeBaseId,
                    document_id: documentId,
                })
            );
        }
    };

    const handlePreviewDocument = (documentId) => {
        setSelectedDocumentId(documentId);
        setPreviewModalVisible(true);
    };

    const handleClosePreviewModal = () => {
        setPreviewModalVisible(false);
        setSelectedDocumentId(null);
    };

    if (loading) {
        return (
            <div className='text-center py-4'>
                <div className='spinner-border text-primary' role='status'>
                    <span className='visually-hidden'>加载中...</span>
                </div>
            </div>
        );
    }

    if (items.length === 0) {
        return (
            <div className='text-center py-4 text-muted'>
                <p>暂无文档，请上传文档</p>
            </div>
        );
    }

    return (
        <>
            <div className='table-responsive'>
                <table className='table table-hover'>
                    <thead className='table-light'>
                        <tr>
                            <th scope='col'>文档名称</th>
                            <th scope='col'>创建时间</th>
                            <th scope='col'>更新时间</th>
                            <th scope='col'>操作</th>
                        </tr>
                    </thead>
                    <tbody>
                        {items.map((doc) => (
                            <tr key={doc.id}>
                                <td>{doc.document_name}</td>
                                <td>{formatDateTime(doc.create_time)}</td>
                                <td>{formatDateTime(doc.update_time)}</td>
                                <td>
                                    <div className='btn-group' role='group'>
                                        <button
                                            className='btn btn-sm btn-outline-dark me-2'
                                            onClick={() => handlePreviewDocument(doc.document_id)}
                                        >
                                            预览
                                        </button>
                                        <button
                                            className='btn btn-sm btn-outline-danger'
                                            onClick={() => handleDeleteDocument(doc.document_id)}
                                        >
                                            删除
                                        </button>
                                    </div>
                                </td>
                            </tr>
                        ))}
                    </tbody>
                </table>

                {pagination.total > 0 && (
                    <div className='d-flex justify-content-between align-items-center mt-3'>
                        <p className='text-muted mb-0'>共 {pagination.total} 条记录</p>
                    </div>
                )}
            </div>

            <DocumentPreviewModal
                show={previewModalVisible}
                documentId={selectedDocumentId}
                knowledgeBaseId={knowledgeBaseId}
                onClose={handleClosePreviewModal}
            />
        </>
    );
};

// Helper function to format date string
const formatDateTime = (dateString) => {
    if (!dateString) return '-';

    // If the utility function exists, use it, otherwise format manually
    try {
        return formatDate(dateString);
    } catch (error) {
        const date = new Date(dateString);
        return date.toLocaleString();
    }
};

export default DocumentList;
